@@ -7,7 +7,7 @@ use anyhow::Result;
 use fuel_asm::Opcode;
 use fuel_gql_client::client::FuelClient;
 use fuel_tx::{
-    Address, AssetId, ContractId, Input, Output, Receipt, StorageSlot, Transaction, UtxoId, Witness,
+    Address, AssetId, ContractId, Input, Output, Receipt, StorageSlot, Transaction, UtxoId,
 };
 use fuel_types::{Bytes32, Salt, Word};
 use fuel_vm::consts::{REG_CGAS, REG_ONE};
@@ -81,17 +81,31 @@ impl Contract {
         external_contracts: Option<Vec<ContractId>>,
         wallet: LocalWallet,
     ) -> Result<Vec<Receipt>, Error> {
-        // Script to call the contract. The offset that points to the `script_data` is loaded at the
-        // register `0x12`. Note that we're picking `0x12` simply because it could be any
-        // non-reserved register. Then, we use the Opcode to call a contract: `CALL` pointing at the
-        // register that we loaded the `script_data` at.
+        // Script to call the contract.
+        // We use the Opcode to call a contract: `CALL` pointing at the
+        // following registers;
+        //
+        // 0x10 Script data offset
+        // 0x11 Gas price  TODO: https://github.com/FuelLabs/fuels-rs/issues/184
+        // 0x12 Coin amount
+        // 0x13 Asset ID
+        //
+        // Note that these are soft rules as we're picking this addresses simply because they
+        // non-reserved register.
+        let forward_data_offset = ContractId::LEN + WORD_SIZE;
         let (script, offset) = script_with_data_offset!(
             data_offset,
             vec![
                 // Load call data to 0x10.
-                Opcode::MOVI(0x10, data_offset + 32),
+                Opcode::MOVI(0x10, data_offset + forward_data_offset as Immediate18),
                 // Load gas forward to 0x11.
-                Opcode::MOVI(0x12, call_parameters.amount as Immediate18),
+                // Load word into 0x12
+                Opcode::MOVI(
+                    0x12,
+                    ((data_offset as usize) + ContractId::LEN) as Immediate18
+                ),
+                // Load the amount into 0x12
+                Opcode::LW(0x12, 0x12, 0),
                 // Load the asset id to use to 0x13.
                 Opcode::MOVI(0x13, data_offset),
                 // Call the transfer contract.
@@ -104,16 +118,22 @@ impl Contract {
 
         // `script_data` consists of:
         // 1. Asset ID to be forwarded
-        // 2. Contract ID (ContractID::LEN);
-        // 3. Function selector (1 * WORD_SIZE);
-        // 4. Calldata offset, if it has structs as input,
+        // 2. Amount to be forwarded
+        // 3. Contract ID (ContractID::LEN);
+        // 4. Function selector (1 * WORD_SIZE);
+        // 5. Calldata offset, if it has structs as input,
         // computed as `script_data_offset` + ContractId::LEN
         //                                  + 2 * WORD_SIZE;
-        // 5. Encoded arguments.
+        // 6. Encoded arguments.
         let mut script_data: Vec<u8> = vec![];
 
+        // Insert asset_id to be forwarded
         script_data.extend(call_parameters.asset_id.to_vec());
 
+        // Insert amount to be forwarded
+        let amount = call_parameters.amount as Word;
+        script_data.extend(amount.to_be_bytes());
+
         // Insert contract_id
         script_data.extend(contract_id.as_ref());
 
@@ -128,7 +148,8 @@ impl Contract {
         // transaction. If it doesn't take any custom inputs, this isn't necessary.
         if compute_calldata_offset {
             // Offset of the script data relative to the call data
-            let call_data_offset = (offset + 32) as usize + ContractId::LEN + 2 * WORD_SIZE;
+            let call_data_offset =
+                ((offset as usize) + forward_data_offset) + ContractId::LEN + 2 * WORD_SIZE;
             let call_data_offset = call_data_offset as Word;
 
             script_data.extend(&call_data_offset.to_be_bytes());
@@ -150,17 +171,39 @@ impl Contract {
         );
         inputs.push(self_contract_input);
 
-        let spendables = wallet
+        let mut spendables = wallet
             .get_spendable_coins(&AssetId::default(), DEFAULT_COIN_AMOUNT as u64)
             .await
             .unwrap();
 
+        // add default asset change if any inputs are being spent
+        if !spendables.is_empty() {
+            let change_output = Output::change(wallet.address(), 0, AssetId::default());
+            outputs.push(change_output);
+        }
+
+        if call_parameters.asset_id != AssetId::default() {
+            let alt_spendables = wallet
+                .get_spendable_coins(&call_parameters.asset_id, call_parameters.amount)
+                .await
+                .unwrap();
+
+            // add alt change if inputs are being spent
+            if !alt_spendables.is_empty() {
+                let change_output = Output::change(wallet.address(), 0, call_parameters.asset_id);
+                outputs.push(change_output);
+            }
+
+            // add alt coins to inputs
+            spendables.extend(alt_spendables.into_iter());
+        }
+
         for coin in spendables {
             let input_coin = Input::coin(
                 UtxoId::from(coin.utxo_id),
                 coin.owner.into(),
                 coin.amount.0,
-                AssetId::default(),
+                coin.asset_id.into(),
                 0,
                 0,
                 vec![],
@@ -175,9 +218,6 @@ impl Contract {
         let self_contract_output = Output::contract(0, Bytes32::zeroed(), Bytes32::zeroed());
         outputs.push(self_contract_output);
 
-        let change_output = Output::change(wallet.address(), 0, AssetId::default());
-        outputs.push(change_output);
-
         // Add external contract IDs to Input/Output pair, if applicable.
         if let Some(external_contract_ids) = external_contracts {
             for (idx, external_contract_id) in external_contract_ids.iter().enumerate() {
@@ -215,7 +255,7 @@ impl Contract {
             script_data,
             inputs,
             outputs,
-            vec![Witness::from(vec![0u8, 0u8])],
+            vec![],
         );
         wallet.sign_transaction(&mut tx).await?;
 
